package me.km.api;

import me.km.permissions.Permission;
import me.km.permissions.Permissions;
import net.minecraft.command.CommandBase;
import net.minecraft.command.CommandException;
import net.minecraft.command.ICommandSender;
import net.minecraft.server.MinecraftServer;

public abstract class ModuleCommand extends CommandBase
{   
    private final Module m;
    private final String name;
    private String usage;
    private String description;
    private Permissions perm;
    
    public ModuleCommand(String name, Module m) 
    {
        this.name = name;
        this.m = m;
        this.usage = "";
        this.perm = null;
    }
    
    public Module getModule()
    {
        return m;
    }
    
    @Override
    public String getName()
    {
        return name;
    }
    
    public void setUsage(String s)
    {
        usage = s;
    }
    
    @Override
    public String getUsage(ICommandSender sender)
    {
        return usage;
    }
    
    public void setDescription(String s)
    {
        description = s;
    }
    
    public String getDescription()
    {
        return description;
    }
    
    public void setPermission(Permissions perm)
    {
        this.perm = perm;
    }
    
    public Permissions getPermission()
    {
        return perm;
    }
    
    public abstract boolean execute(ICommandSender cs, String[] arg);
    
    @Override
    public void execute(MinecraftServer server, ICommandSender cs, String[] args) throws CommandException
    {
        if(!Permission.hasPermission(cs, this))
        {
            m.sendWarning(cs, "Du hast keine Rechte für diesen Command.");
            return;
        }       
        if(!execute(cs, args))
        {
            m.send(cs, this.getUsage(cs));
        }
    }

    @Override
    public boolean checkPermission(MinecraftServer server, ICommandSender cs) 
    {
        if(Permission.hasPermission(cs, this))
        {
            return true;
        }  
        m.sendWarning(cs, "Du hast keine Rechte für diesen Command. 2");
        return false;
    }
}
