package me.km.scoreboard;

import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.HashSet;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.scoreboard.IScoreCriteria;
import net.minecraft.scoreboard.Score;
import net.minecraft.scoreboard.ScoreObjective;
import net.minecraft.scoreboard.Scoreboard;

public class ScoreboardData 
{
    private final HashMap<Integer, String> elements;
    private final Scoreboard board;
    private final ScoreObjective o;
    private final HashSet<Integer> toRemove;
    private boolean changeNeeded;
    
    public ScoreboardData()
    {
        toRemove = new HashSet<>();
        elements = new HashMap<>();
        changeNeeded = true;
        board = new Scoreboard();
        o = board.addScoreObjective("ScoreBoardAPI", IScoreCriteria.DUMMY);
        o.setDisplayName("§6---------------");
        board.setObjectiveInDisplaySlot(1, o);
    }
    
    private void add(String s, int id)
    {
        board.getOrCreateScore(s, o).setScorePoints(id);
    }
      
    private void resetScores(String entry)
    {
        this.board.getScoreObjectives().stream().forEach((ob) -> 
        {
            board.removeObjectiveFromEntity(entry, ob);
        });
    }
    
    public void update(EntityPlayer p)
    {         
        if(!changeNeeded)
        {
            return;
        }
        
        if(!toRemove.isEmpty())
        {
            ArrayList<String> remove = new ArrayList<>();
            board.getObjectiveNames().stream().forEach((s) -> 
            {
                Score score = board.getOrCreateScore(s, o);
                if(score != null && toRemove.contains(score.getScorePoints())) 
                {
                    remove.add(s);
                }
            });
            remove.forEach(s -> resetScores(s));
        }

        toRemove.clear();
        
        elements.entrySet().stream().forEach((e) -> 
        {
            add(e.getValue(), e.getKey());
        });      
        
        changeNeeded = false;
        //TODO
        //p.setScoreboard(board);
    }

    public void addText(int id, String text)
    { 
        String s = elements.put(id, text);
        if(s != null && !s.equals(text))
        {
            toRemove.add(id);
            changeNeeded = true;
        }
        else if(s == null)
        {
            changeNeeded = true;
        }
    }
    
    public void removeText(int id)
    { 
        if(elements.remove(id) != null)
        {
            toRemove.add(id);
            changeNeeded = true;
        }
    }
}
